function [] = deconv_Time_Series_Data()

%% deconv_Time_Series_Data - FUNCTION to perform deconvolution of time series stack. To be used after align_PSF_DataStack
% A defined set of slices in the data stack are deconvolved by a defined
% set of 2D PSF images sample across z-axis.
% Output is a series of deconvolved images which are then manually analyzed
% to determine best global alignment of the PSF with the raw data stack.
% Author: Raju Tomer (rajutomer@gmail.com)
%%

%%
%%%%%%%%% Parameters %%%%%%%%%%%
no_of_threads = 12; % no. of threads to use for deconvolution calculations
base_dir = 'D:\SPED_data\Exp1';
out_dir = [base_dir '\LOC000_dc_noMedFilt'];
data_dir = [base_dir '\LOC000'];
psf_fname = [base_dir '\O4xPSF_AvgBeadV2_AvgOf10imagesLo40_Hi100b65k_Cent_114Rev_073m1.tif'];
data_ind_arr = 1:39; % this and the parameter below, psf_ind_arr defined the z-axis mapping of data and PSF indices
psf_ind_arr = 170:5:360; % ""
stack_size = [2048 316 40]; % individual data stack size informations, needed as images were stored in raw binary format
start_TP = 0; % range of time points to process
stop_TP = 4000; % range of time points to process
N_IT_arr = [2 5 10 15]; % vector containing number of iterations to use for Richardson Lucy deconvolution
camera_bg = 100; % empty image background
doMedFiltering = 0; % whether to do median filtering
doCropping = 0; % wether to crop the data, if 1 then cropping_rect should be defined
cropping_rect = []; %a 4-element vector with the form [XMIN YMIN XMAX YMAX] from imageJ
upsample_im = 0; % if to upsample data before deconvolution
upsample_scale = 2; % if above is 1, the umsampling factor
write_mip=1; % whether to write maximum intensity projection image for deconvolved time points
tiff_sw = 0; % if the data is in tiff stack (1) or in raw (binary format (0)
reverse_stack = 1; % 1 for reversing it (whether to revere the stack slice order
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%

if (numel(data_ind_arr) ~= numel(psf_ind_arr))
    disp('Number of Indices for PSF and data stack do not match, check that');
    return;
end

if (doCropping > 0)
    cropping_rect = cropping_rect + 1;
end

parpool(no_of_threads);

if (tiff_sw > 0)
    list_TPs = dir([data_dir '\*.tif*']);
else
    list_TPs = dir([data_dir '\*.stack*']);
end

kernel_for_PSF_taper = fspecial('gaussian',[20,20],4);

parfor tp = (start_TP + 1):(stop_TP + 1)
    fname_TP = [data_dir '\' list_TPs(tp).name];
    im = [];
    if (tiff_sw == 1)
        im = zeros(stack_size, 'uint16');
        for r = 1:stack_size(3)
            im(:,:,r) = imread(fname_TP,r);
        end
    else    
        fid = fopen(fname_TP, 'r', 'l' );
        im = fread( fid, prod(stack_size), 'uint16' );
        fclose( fid );
        im = reshape(im,stack_size);
        im = permute(im, [2 1 3]);
    end
    if (reverse_stack == 1)
        im = flip(im,3);
    end
    if (doCropping > 0)
        im = im(cropping_rect(2):cropping_rect(4), cropping_rect(1):cropping_rect(3),:)
    end
    for N_IT = N_IT_arr
        ofname = [out_dir '\DC' num2str(N_IT) '_' list_TPs(tp).name '.tif'];
        if (exist(ofname, 'file') == 2)
            disp('skipping');
            ofname
            continue
        end
        ofname_mip = [out_dir '\MAX_DC' num2str(N_IT) '_' list_TPs(tp).name '.tif'];
        ofname_mip_tmp = [out_dir '\Tmp_MAX_DC' num2str(N_IT) '_' list_TPs(tp).name '.tif'];
        first_slice=1;
        tic
        for i = 1:numel(data_ind_arr)
            data_ind = data_ind_arr(i);
            psf_ind = psf_ind_arr(i);    
            data = double(im(:,:,data_ind));
            data = data - camera_bg;
            data(data < 0) = 0;
            if (doMedFiltering > 0)
                data = medfilt2(data);
            end
            if (upsample_im == 1)
                disp('UpSampling Data');
                data = imresize(data, upsample_scale, 'bicubic');
                disp('UpSampling Data Done');
            end
            psf = imread(psf_fname, psf_ind);
            psf = psf - min(psf(:));
            psf = double( psf) ./ sum(sum( double(psf)));
            psf = edgetaper( psf, kernel_for_PSF_taper);
            psf = padarray( psf, [20,20] );
            data_ind
            N_IT
        	restoration = deconvlucy( data, psf, N_IT );
            restoration = restoration - min(min(restoration));
            imwrite(uint16(restoration), ofname, 'tiff', 'Compression', 'none', 'WriteMode', 'append');
            if (first_slice == 1)
                max_im = restoration;
                first_slice = 0;
            else
                max_im = max(restoration, max_im);
            end            
            if (write_mip == 1)
                imwrite(uint16(max_im), ofname_mip_tmp, 'tiff', 'Compression', 'None');
            end
        end
        toc
        if (write_mip == 1)
            imwrite(uint16(max_im), ofname_mip, 'tiff', 'Compression', 'None');
            delete(ofname_mip_tmp);
        end
    end
end 
delete(gcp('nocreate'));
